@@ -169,7 +169,7 @@ URL_GHUGENT_HPCUGENT = 'https://github.ugent.be/hpcugent/%(name)s'
 
 RELOAD_VSC_MODS = False
 
-VERSION = '0.17.21'
+VERSION = '0.17.28'
 
 log.info('This is (based on) vsc.install.shared_setup %s' % VERSION)
 log.info('(using setuptools version %s located at %s)' % (setuptools.__version__, setuptools.__file__))
@@ -1493,6 +1493,8 @@ class vsc_setup(object):
             tests_requires.append('pyyaml >= 5.4.1, < 6.0')
             # lazy_object_proxy 1.7.0 no longer compatible with python 2
             tests_requires.append('lazy_object_proxy < 1.7.0')
+            # requirements-detector 1.0.0 no longer compatible with python 2
+            tests_requires.append('requirements-detector < 1.0.0')
         else:
             # soft pinning of (transitive) dependencies of prospector
             # ('~=' means stick to compatible release, https://www.python.org/dev/peps/pep-0440/#compatible-release);
@@ -1500,18 +1502,34 @@ class vsc_setup(object):
             # - https://github.com/PyCQA/pylint/blob/v2.12.2/setup.cfg
             # - https://github.com/PyCQA/flake8/blob/3.9.2/setup.cfg
             # - https://github.com/PyCQA/prospector/blob/1.5.3.1/pyproject.toml
+
+            # To figure out requirements of what needs what: grep name_of_tool .eggs.py3/*/*/requires.txt
             tests_requires.extend([
-                'pyflakes~=2.3.0',
-                'pycodestyle~=2.7.0',
-                'pylint~=2.12.2',
-                'flake8~=3.9.2',
-                'prospector~=1.5.3.1',
-                'pylint-plugin-utils < 0.7',
-                'pylint-django~=2.4.4',
-                # platformdirs >= 2.4.0 requires Python 3.7, use older versions for running tests with Python 3.6
-                'platformdirs < 2.4.0',
                 'mock',
             ])
+            if sys.version_info < (3, 7):
+                tests_requires.extend([
+                    'pyflakes~=2.3.0',
+                    'pycodestyle~=2.7.0',
+                    'pylint~=2.12.2',
+                    'prospector~=1.5.3.1',
+                    'flake8~=3.9.2',
+                    'pylint-plugin-utils < 0.7',
+                    'pylint-django~=2.4.4',
+                    # platformdirs >= 2.4.0 requires Python 3.7, use older versions for running tests with Python 3.6
+                    'platformdirs < 2.4.0',
+                    'typing-extensions < 4.2.0', # higher requires python 3.7
+                    'lazy-object-proxy < 1.8.0', # higher requires python 3.7
+                    'jsonpickle < 3.0.0', # higher requires python 3.7
+                ])
+            else:  # tested for fedora37 py3.11
+                tests_requires.extend([
+                    'flake8 < 5.0.0',
+                    'astroid <= 2.12.0-dev0',
+                    'pyflakes < 2.5.0',
+                    'pylint~=2.14.4',
+                    'prospector~=1.7.7',
+                ])
 
         new_target['tests_require'] = tests_requires
 
