@@ -1,5 +1,5 @@
 #
-# Copyright 2012-2021 Ghent University
+# Copyright 2012-2022 Ghent University
 #
 # This file is part of vsc-base,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -42,11 +42,11 @@ Various functions that are missing from the default Python library.
 import logging
 import shlex
 import time
-from collections import namedtuple, Mapping
+from collections import namedtuple
 from functools import reduce
 
 from vsc.utils.frozendict import FrozenDict
-from vsc.utils.py2vs3 import quote
+from vsc.utils.py2vs3 import quote, Mapping
 
 
 def nub(list_):
@@ -318,6 +318,8 @@ class TryOrFail(object):
                         logging.warning("try_or_fail is sleeping for %d seconds before the next attempt", self.sleep)
                         time.sleep(self.sleep)
 
+                return None
+
         return new_function
 
 
